// support vectors of integers, clones from piplib's charvec

#include "intvec.h"

static const char rcsid[]=
"$Id$";

/* This implementation uses a "growable array" abstraction */

int intvec_fini(intvec_t *t)
{
    if (t->a && t->free) { t->free(t->a); t->a = 0; t->max = 0; }
    t->len = 0;
    return 1;
}

#ifndef BASE_ALLOC
#define BASE_ALLOC 30
#endif
enum { BASE = BASE_ALLOC };

int intvec_need(intvec_t *t, unsigned int n)
{
    if (t->a == 0) {
        t->len = 0; 
        t->max = n;
        t->a = t->alloc(0, n * sizeof(int));
        return t->a != 0;
    }

    if (n > t->max) { 
        unsigned int i = BASE + n + (n >> 3); 
        void *p = t->alloc(t->a, i * sizeof(int));
        if (!p)
            return 0;
        t->max = i;
        t->a = p;
    }
    return 1;
}

int intvec_more(intvec_t *t, unsigned int n)
{
    return intvec_need(t, n + t->len);
}

int intvec_append(intvec_t *t, int e)
{
    if (!intvec_more(t, 1))
        return 0;
    t->a[t->len++] = e;
    return 1;
}

int intvec_fit(intvec_t *t)
{
    unsigned int i = t->len;
    void *p = t->alloc(t->a, i * sizeof(int));
    if (!p)
        return 0;
    t->max = i;
    t->a = p;
    return 1;
}

